@@ -46,7 +46,9 @@ module Text.Parser.Char
   , CharParsing(..)
   ) where
 
+#if __GLASGOW_HASKELL__ < 710
 import Control.Applicative
+#endif
 import Control.Monad.Trans.Class
 import Control.Monad.Trans.State.Lazy as Lazy
 import Control.Monad.Trans.State.Strict as Strict
@@ -62,7 +64,9 @@ import Data.CharSet (CharSet(..))
 import qualified Data.CharSet as CharSet
 import Data.Foldable
 import qualified Data.IntSet as IntSet
+#if __GLASGOW_HASKELL__ < 710
 import Data.Monoid
+#endif
 import Data.Text
 import qualified Text.ParserCombinators.ReadP as ReadP
 import qualified Text.Parsec as Parsec
@@ -81,7 +85,7 @@ oneOf xs = oneOfSet (CharSet.fromList xs)
 {-# ANN oneOf "HLint: ignore Use String" #-}
 
 -- | As the dual of 'oneOf', @noneOf cs@ succeeds if the current
--- character /not/ in the supplied list of characters @cs@. Returns the
+-- character is /not/ in the supplied list of characters @cs@. Returns the
 -- parsed character.
 --
 -- >  consonant = noneOf "aeiou"
@@ -101,7 +105,7 @@ oneOfSet (CharSet False _ is) = satisfy (\c -> not (IntSet.member (fromEnum c) i
 {-# INLINE oneOfSet #-}
 
 -- | As the dual of 'oneOf', @noneOf cs@ succeeds if the current
--- character /not/ in the supplied list of characters @cs@. Returns the
+-- character is /not/ in the supplied list of characters @cs@. Returns the
 -- parsed character.
 --
 -- >  consonant = noneOf "aeiou"
